{========================================================
 PASMUL - PASMulator Version 0.1
 =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 Real mode Z80 arcade emulator
=========================================================}

{$F+}
{$O+}

uses
  Overlay,
  UZ80Var,
  UKBD,
  Detect,
  SMix,
  Dos,
  ULoadRom,
  UGraph,
  UStrings,
  XLib2,
  uz80,
  UObjects,
  {DRIVERS} UDefault, UGalax, UInvad, UPhoenix, UFrogger;

{$O UGalax}
{$O UInvad}
{$O UPhoenix}
{$O UFrogger}



var
  oDebug: Text;
  aPFont: Pointer;

procedure DoneGraphics;
begin
  xMouseRemove;
  xTextMode;
end;

procedure InitGraphics;
var
  I: Integer;
begin
  GetMaxX := 360;
  GetMaxY := 270;
  nYPos := (GetMaxY-32*8) div 2;
  xSetMode(XMODE360x270, 360);
  xSetDoubleBuffer(270);

  for i := 0 to 255 do
    with aPalette[i] do
      xSetRGB(i, R shr 2, G shr 2, B shr 2);

  xMouseInit;
  xMouseWindow(0, 0, ScrnLogicalPixelWidth, ScrnLogicalHeight);
  xHideMouse;
  MouseColor := nWhite;

  xTextInit;
  xRegisterUserFont(aPFont^);
  xSetFont(2);
end;

procedure ExitProgram(cMessage1, cMessage2: String);
var
  i: Integer;
begin
  if lSound then begin
    ShutdownMixing;
    ShutdownSB;
    for i := 0 to 8 do
      FreeSound(Sound[i]);
    ShutdownSharing;
  end;

  Writeln(cMessage1);
  Writeln(cMessage2);
  Halt;
end;

function ExtractTitle(cFileName: String): String;
var
  F: Text;
  cLine: String;
  lAchou: Boolean;
begin
  FileMode := 0;
  lAchou := false;
  Assign(F, cFileName);
  Reset(F);
  while not Eof(F) do begin
    Readln(F, cLine);
    if Copy(cLine, 1, 7)='#TITLE=' then begin
      Delete(cLine, 1, 7);
      lAchou := true;
      break;
    end;
  end;

  if lAchou then
    ExtractTitle := cLine
  else
    ExtractTitle := cFileName;

  Close(F);
end;


var
  I: Word;
  F: File;
  nCont: LongInt;
  nSize: Word;
  nSprite: Integer;
  nLinSpr,
  nColSpr,
  nEnder: LongInt;
  J,
  nPixColor: Integer;
  oLBMCursor: PLBMImage;
  nColor: LongInt;
  nCharSet: Integer;
  S: SearchRec;
  cAux: String;
  lInvertY,
  lInvertX: Boolean;
  nDeltaPlane,
  nDeltaCharSet: LongInt;
  BaseIO: word;    { Sound Blaster Hardware Info }
  IRQ,
  DMA,
  DMA16: byte;
  nPage: Word;
  nMemUsed: LongInt;

function XY(nLinSpr, nColSpr, nDY, nDX: Integer): Integer;
begin
  if lInvertX then nColSpr := 7-nColSpr;
  if lInvertY then nLinSpr := 7-nLinSpr;
  XY := (nLinSpr+nDY)*16+nColSpr+nDX;
end;

procedure CreateDefaultCharSet;
var
  oAux: PLBMImage;
  nColor: Integer;
  oSprites: record
    nWidth, nHeight: Byte;
    aData: array[1..8, 1..8] of Byte;
  end;

  function Conv(I: Byte): Byte;
  var
    J: Byte;
  begin
    J := I;
    J := (J and $1) shl 1;
    I := (I and $2) shr 1;
    Conv := J or I
  end;

begin
  MapSystemColor;

  for nCharSet := 0 to oROMSInfo.nCharSet-1 do begin
    oSprites.nWidth := 8;
    oSprites.nHeight := 8;
    for nSprite := 0 to 255 do begin
      nColor := nSprite shr 5;
      for nLinSpr := 0 to 7 do begin
        for nColSpr := 0 to 7 do begin
          nEnder := oROMsInfo.nGRFDecode+(nSprite*8+nLinSpr)+nCharSet*nDeltaCharSet;
          nPixColor := Bit(aZ80Mem^[nEnder+nDeltaPlane], nColSpr);
          nPixColor := (nPixColor shl 0) or
                       (Bit(aZ80Mem^[nEnder+$0000], nColSpr) shl 1);
          nColor := nPixColor;
          if oRomsInfo.nDrvTag=$50 then
            nColor := Conv(nColor);


          if lInvertX then
            if lInvertY then
              oSprites.aData[7-nColSpr+1, 7-nLinSpr+1] := nColor
            else
              oSprites.aData[7-nColSpr+1, nLinSpr+1] := nColor
          else
            if lInvertY then
              oSprites.aData[nColSpr+1, 7-nLinSpr+1] := nColor
            else
              oSprites.aData[nColSpr+1, nLinSpr+1] := nColor
        end;
        GetMem(aLBMChars[nSprite+nCharSet*256], 8*8);
        Move(oSprites.aData, aLBMChars[nSprite+nCharSet*256]^, 8*8);
      end;
    end;
  end;

  if oROMSInfo.nSprites<>0 then begin
    GetMem(oAux, 2+16*16);
    oAux^.nW := 16;
    oAux^.nH := 16;

    for i := 0 to oROMSInfo.nSprites-1 do begin
      nSprite := 4*i;
      nCharSet := nSprite div 256;
      nSprite := nSprite mod 256;

      for nLinSpr := 0 to 7 do
        for nColSpr := 0 to 7 do begin
          nEnder := oROMsInfo.nGRFDecode+((nSprite+0)*8+nLinSpr)+nCharSet*nDeltaCharSet;
          nPixColor := Bit(aZ80Mem^[nEnder+nDeltaPlane], nColSpr);
          nPixColor := (nPixColor shl 0) or
                       (Bit(aZ80Mem^[nEnder+$0000], nColSpr) shl 1);
          nColor := nPixColor;
          if oRomsInfo.nDrvTag=$50 then
            nColor := Conv(nColor);

          oAux^.aData[XY(nColSpr, nLinSpr, 0, 8)] := nColor;
        end;

      for nLinSpr := 0 to 7 do
        for nColSpr := 0 to 7 do begin
          nEnder := oROMsInfo.nGRFDecode+((nSprite+1)*8+nLinSpr)+nCharSet*nDeltaCharSet;
          nPixColor := Bit(aZ80Mem^[nEnder+nDeltaPlane], nColSpr);
          nPixColor := (nPixColor shl 0) or
                       (Bit(aZ80Mem^[nEnder+$0000], nColSpr) shl 1);
          nColor := nPixColor;
          if oRomsInfo.nDrvTag=$50 then
            nColor := Conv(nColor);
          oAux^.aData[XY(nColSpr, nLinSpr, 8, 8)] := nColor;
        end;

      for nLinSpr := 0 to 7 do
        for nColSpr := 0 to 7 do begin
          nEnder := oROMsInfo.nGRFDecode+((nSprite+2)*8+nLinSpr)+nCharSet*nDeltaCharSet;
          nPixColor := Bit(aZ80Mem^[nEnder+nDeltaPlane], nColSpr);
          nPixColor := (nPixColor shl 0) or
                       (Bit(aZ80Mem^[nEnder+$0000], nColSpr) shl 1);
          nColor := nPixColor;
          if oRomsInfo.nDrvTag=$50 then
            nColor := Conv(nColor);
          oAux^.aData[XY(nColSpr, nLinSpr, 0, 0)] := nColor;
        end;

      for nLinSpr := 0 to 7 do
        for nColSpr := 0 to 7 do begin
          nEnder := oROMsInfo.nGRFDecode+((nSprite+3)*8+nLinSpr)+nCharSet*nDeltaCharSet;
          nPixColor := Bit(aZ80Mem^[nEnder+nDeltaPlane], nColSpr);
          nPixColor := (nPixColor shl 0) or
                       (Bit(aZ80Mem^[nEnder+$0000], nColSpr) shl 1);
          nColor := nPixColor;
          if oRomsInfo.nDrvTag=$50 then
            nColor := Conv(nColor);
          oAux^.aData[XY(nColSpr, nLinSpr, 8, 0)] := nColor;
        end;

      GetMem(aSprites[i], 16*16);
      Move(oAux^.aData, aSprites[i]^, 16*16);
    end;
    FreeMem(oAux, 2+16*16);
  end;
end;


var
  cGameName: String;
  lHasGame: Boolean;
  oCheckBox: PCheckBox;
  oCancButton,
  oOkButton: PButton;
  oListBox: PListBox;
  lExit: Boolean;
  lFimMenu: Boolean;
  L, K: Integer;
begin
  Assign(oDebug, 'debug.log');
  Rewrite(oDebug);

  OvrInit('PASMUL.OVR');
  if OvrResult <> ovrOk then begin
    case OvrResult of
      ovrError: Writeln('Program has no overlays.');
      ovrNotFound: Writeln('Overlay file not found.');
    end;
    Halt(1);
  end;

  for i := 0 to 255 do begin
    aPalette[i].R := ((Palette16[i mod 16].R+1) shl 2)-1 ;
    aPalette[i].G := ((Palette16[i mod 16].G+1) shl 2)-1 ;
    aPalette[i].B := ((Palette16[i mod 16].B+1) shl 2)-1 ;
  end;
  nBlack := 16;
  aPalette[16].R := 0;
  aPalette[16].G := 0;
  aPalette[16].B := 0;

  {$I-}
  Assign(F, 'fixed6x8.fnt');
  Reset(F,1);
  {$I+}
  if IOResult<>0 then begin
    Writeln('I need the font file (fixed6x8.fnt).');
    Halt(1);
  end;

  nSize := FileSize(f);
  GetMem(aPFont, nSize);
  BlockRead(F, aPFont^, nSize);
  Close(F);

  LoadLBM(oButtonSprite[OK_SPRITE], 'ok.lbm');
  LoadLBM(oButtonSprite[CANCEL_SPRITE], 'cancel.lbm');
  LoadLBM(oButtonSprite[EXIT_SPRITE], 'exit.lbm');
  LoadLBM(oButtonSprite[UP_SPRITE], 'setacima.lbm');
  LoadLBM(oButtonSprite[DOWN_SPRITE], 'setabaix.lbm');
  LoadLBM(oButtonSprite[CHECK1_SPRITE], 'check1.lbm');
  LoadLBM(oButtonSprite[CHECK2_SPRITE], 'check2.lbm');
  LoadLBM(oButtonSprite[RADIO1_SPRITE], 'radio1.lbm');
  LoadLBM(oButtonSprite[RADIO2_SPRITE], 'radio2.lbm');

  lHasGame := false;
  cGameName := '';
  for i := 1 to ParamCount do begin
    cAux := ParamStr(i);
    if cAux[1]<>'-' then begin
      cGameName := cAux+'.ini';
      lHasGame := true;
    end;
  end;


  if not lHasGame then begin
    InitGraphics;
    oKBD.Init;
    xRectFill(0, 0, GetMaxX, GetMaxY, VISIBLEPAGEOFFS, nLightBlue);
    xPrintf(4+2, 08+3, VISIBLEPAGEOFFS, nDarkGray, 'PASMULATOR - 0.1');
    xPrintf(4+2, 08+2, VISIBLEPAGEOFFS, nDarkGray, 'PASMULATOR - 0.1');
    xPrintf(4+3, 08+3, VISIBLEPAGEOFFS, nDarkGray, 'PASMULATOR - 0.1');
    xPrintf(4+1, 08+1, VISIBLEPAGEOFFS, nYellow, 'PASMULATOR - 0.1');

    xPrintf(4+1, GetMaxY-12, VISIBLEPAGEOFFS, nYellow, 'April 1998, by pscardoso@lme.usp.br');

    oListBox := New(PListBox, Init(0, 4, 020, 350, 215, kEsc));
    oOkButton := New(PButton, Init(1, 004, 237, 70, 20, kEnter, 'Start', oButtonSprite[OK_SPRITE]));
    oCancButton := New(PButton, Init(1, 080, 237, 70, 20, kEsc, 'Exit', oButtonSprite[EXIT_SPRITE]));
    oCheckBox := New(PCheckBox, Init(1, 155, 237, 50, 20, kS, 0, 'Sound',
      oButtonSprite[CHECK1_SPRITE], oButtonSprite[CHECK2_SPRITE]));

    FindFirst('*.ini', ANYFILE, S);
    while DOSError=0 do begin
      oListBox^.Add(ExtractTitle(S.Name)+#1+S.Name);
      FindNext(S);
    end;

    xShowMouse;
    lFimMenu := false;
    lExit := false;
    oListBox^.Draw;
    oOkButton^.Draw;
    oCancButton^.Draw;
    oCheckBox^.Draw;
    repeat
      if oKBD.Key(kf6) then begin
        while oKBD.Key(kf6) do;
        SaveGIFProc;
      end;
      if oCheckBox^.CheckMessage then begin
      end;
      if oListBox^.CheckMessage or oOkButton^.CheckMessage then begin
        lFimMenu := true;
      end;
      if oCancButton^.CheckMessage then begin
        lFimMenu := true;
        lExit := true;
      end;
    until lFimMenu;
    cGameName := oListBox^.GetString;
    Dispose(oListBox, Done);
    Dispose(oOkButton, Done);
    Dispose(oCancButton, Done);
    Dispose(oCheckBox, Done);
    xHideMouse;

    DoneGraphics;
    oKBD.Done;
  end;




  if not lExit then begin
    Writeln('Loading arcade info for '+cGameName);
    oROMSInfo.LoadInfo(cGameName);
    Writeln('Game name is '+oROMSInfo.cTitle+', working with '+oROMSInfo.cDriver+' driver.');
  end;


  lSound := false;
  if not lExit then begin
    lSound := false;
    if lSound then begin
      if not(GetSettings(BaseIO, IRQ, DMA, DMA16)) then begin
        writeln('Error initializing:  Invalid or non-existant BLASTER environment variable');
        Halt(1)
      end;

      if not(InitSB(BaseIO, IRQ, DMA, DMA16)) then begin
        writeln('Error initializing sound card');
        writeln('Incorrect base IO address, sound card not installed, or broken');
        Halt(2); {Sound card could not be initialized}
      end;

      if not(InitXMS) then begin
        xMouseRemove;
        xTextMode;
        writeln('Error initializing extended memory');
        writeln('HIMEM.SYS must be installed');
        Halt(3); {XMS driver not installed}
      end;

      InitSharing;
      OpenSoundResourceFile('invaders\invaders.snd');
      LoadSound(Sound[0], '0');
      LoadSound(Sound[1], '1');
      LoadSound(Sound[2], '2');
      LoadSound(Sound[3], '3');
      LoadSound(Sound[4], '4');
      LoadSound(Sound[5], '5');
      LoadSound(Sound[6], '6');
      LoadSound(Sound[7], '7');
      LoadSound(Sound[8], '8');
      CloseSoundResourceFile;
      InitMixing;
    end;

    if oROMSInfo.nError<>0 then begin
      ExitProgram(oROMSInfo.cErrorStr1, oROMSInfo.cErrorStr2);
    end;

    if MaxAvail<oROMsInfo.nMemory then begin
      Writeln('No sufficient memory to execute the emulator.');
      Writeln('Abnormal program termination.');
      Halt(1);
    end;
    GetMem(aZ80Mem, oROMsInfo.nMemory);
    FillChar(aZ80Mem^, oROMsInfo.nMemory, 0);

    aZ80Regs := @aProcs;
    FillChar(aProcs, SizeOf(aProcs), 0);
    aProcs[1].nIM := 0;
    aProcs[1].nHalted := 0;
    aProcs[1].nPendingNMI := 1;
    aZ80Regs^.IFF1 := 1;
    aZ80Regs^.IFF2 := 1;
    aZ80Regs^.SP.W := $4F00;
    oROMSInfo.LoadRoms(aZ80Mem, nil);

    lInvertY := false;
    lInvertX := false;
    nDeltaPlane := $800;
    nDeltaCharSet := 4096;

    if oROMSInfo.cDriver='INVADERS' then begin
      aProcs[1].oGetByteFunc := InvadersGetByte;
      aProcs[1].oPutByteFunc := InvadersPutByte;
      aProcs[1].oOutByteFunc := InvadersOutByte;
      aProcs[1].oInByteFunc  := InvadersInByte;
      aProcs[1].oInterruptFunc := InvadersInterrupt;
      aProcs[1].oMemory := aZ80Mem;
      aProcs[1].nMaxRAM := $5FFF;
      aProcs[1].nNMIInt := $0010;
    end
    else if oROMSInfo.cDriver='PHOENIX' then begin
      aProcs[1].oGetByteFunc := PhoenixGetByte;
      aProcs[1].oPutByteFunc := PhoenixPutByte;
      aProcs[1].oOutByteFunc := NullOutByte;
      aProcs[1].oInByteFunc  := NullInByte;
      aProcs[1].oInterruptFunc := PhoenixInterrupt;
      aProcs[1].oMemory := aZ80Mem;
      aProcs[1].nMaxRAM := $4FFF;
      lInvertY := true;
      lInvertX := false;
      PhoenixConvertColorPROM;
      CreateDefaultCharSet;
    end
    else if oROMSInfo.cDriver='GALAXIAN' then begin
      aProcs[1].oGetByteFunc := GalaxianGetByte;
      aProcs[1].oPutByteFunc := GalaxianPutByte;
      aProcs[1].oOutByteFunc := NullOutByte;
      aProcs[1].oInByteFunc  := NullInByte;
      aProcs[1].oInterruptFunc := GalaxianInterrupt;
      aProcs[1].oMemory := aZ80Mem;
      aProcs[1].nMaxRAM := $4FFF;
      aProcs[1].nNMIInt := $0066;

      if oRomsInfo.nDrvTag=$50 then begin
        nDeltaCharSet := $0800;
        nDeltaPlane := $1000;
      end
      else begin
        nDeltaCharSet := $1000;
        nDeltaPlane := $800;
      end;

      lInvertY := true;
      lInvertX := true;
      GalaxianConvertColorPROM(oROMsInfo.cColorPROM);
      CreateDefaultCharSet;
    end
    else if oROMSInfo.cDriver='FROGGER' then begin
      aProcs[1].oGetByteFunc := FroggerGetByte;
      aProcs[1].oPutByteFunc := FroggerPutByte;
      aProcs[1].oOutByteFunc := NullOutByte;
      aProcs[1].oInByteFunc  := FroggerInByte;
      aProcs[1].oInterruptFunc := FroggerInterrupt;
      aProcs[1].oMemory := aZ80Mem;
      aProcs[1].nMaxRAM := $4FFF;
      aProcs[1].nNMIInt := $0066;
      nDeltaCharSet := $1000;
      lInvertY := true;
      lInvertX := true;
      {GalaxianConvertColorPROM(oROMsInfo.cColorPROM);}
      CreateDefaultCharSet;
    end
    else begin
      ExitProgram('Invalid Driver! '+oROMSInfo.cDriver, '');
      Halt(1);
    end;

    nXPos := (GetMaxX-oROMSInfo.nVW*8) div 2;


    InitGraphics;
    oKBD.Init;
    xRectFill(0, 0, GetMaxX, GetMaxY, VISIBLEPAGEOFFS, nBLACK);

    while not lStopEmulation do begin
      if lDumpReg then begin
        Writeln(oDebug,  'PC='+ToHex(aZ80Regs^.PC.W, 4)+
                         ' AF='+ToHex(aZ80Regs^.AF.W, 4)+
                         ' BC='+ToHex(aZ80Regs^.BC.W, 4)+
                         ' DE='+ToHex(aZ80Regs^.DE.W, 4)+
                         ' HL='+ToHex(aZ80Regs^.HL.W, 4)+
                         ' IX='+ToHex(aZ80Regs^.IX.W, 4)+
                         ' IY='+ToHex(aZ80Regs^.IY.W, 4)+
                         ' SP='+ToHex(aZ80Regs^.SP.W, 4)+
                         ' [PC+1]='+ToHex((aZ80Mem^[aZ80Regs^.PC.W+1]), 2)+
                         ' [PC]='+ToHex((aZ80Mem^[aZ80Regs^.PC.W]), 2)+
                         ' [HL]='+ToHex((aZ80Mem^[aZ80Regs^.HL.W]), 2)+
                         ' [SP]='+ToHex((aZ80Mem^[aZ80Regs^.SP.W+1]), 2)+ToHex((aZ80Mem^[aZ80Regs^.SP.W]), 2)
                          );
        EvalOneInstr(1);
      end
      else
        EvalInstr(1);
    end;
    FreeMem(aZ80Mem, oROMsInfo.nMemory);
    oRomsInfo.Free;
    oKBD.Done;
    DoneGraphics;
  end;


  FreeMem(aPFont, nSize);
  Close(oDebug);

  OvrClearBuf;
  for i := 1 to MAX_BUTTON_SPRITES do
    freeLBM(oButtonSprite[i]);
  ExitProgram('Bye, Bye...', '');
end.