{=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 Unit UCharSet
 -------------
 Show all the char sets and sprites used by the driver.

 Paulo Sergio Cardoso - April 7, 1998
 =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=}
unit UCharSet;

interface

 procedure ShowCharSet(lSprite: Boolean);

implementation

uses
  XLIB2, UGraph, UZ80Var, UKBD, UStrings, UObjects, UDefault;

  const
    aTypes: array[0..3] of String[8] =
    ('NOT FLIP', 'FLIP X', 'FLIP Y', 'FLIP XY');

  procedure ShowCharSet(lSprite: Boolean);
  const
    nW = 256;  { Width of the Window }
    nH = 200;  { Height of the Window }
  var
    nType,         { Type: 0 not flipped, 1 flip X, 2 flip Y, 3 flip XY }
    nX,            { Left position of the windows in the screen }
    nY,            { Top position of the windows in the screen }
    nColor,        { Color of the sprite }
    nCharSet,      { Number of the charset }
    I, j: Integer;
    oButtons: array[1..16] of PParent;
    lExit, lRedraw: Boolean;
    P: PPointer;
    nDSpr, nCharSets, nMaxSpr, nMod, nWSpr, nHSpr, nWDrw, nHDrw: Integer;
  begin
    nCharSet := 0;   { The first charset }
    nColor := 0;     { The first color }
    nType := 0;      { Doesn't flip the sprite }
    nX := (GetMaxX-nW) div 2;  { Center the window in the screen }
    nY := (GetMaxY-nH) div 2;

    if lSprite then begin
      if oROMsInfo.nSprites=0 then Exit;
      P := @aSprites;
      nWSpr := 16;
      nHSpr := 16;
      nWDrw := 20;
      nHDrw := 20;
      nMod := 09;
      nMaxSpr := 64;
      nDSpr := 64;
      nCharSets :=  oROMsInfo.nSprites div 64;
    end
    else begin
      if oROMsInfo.nCharSet=0 then Exit;
      P := @aLBMChars;
      nWSpr := 8;
      nHSpr := 8;
      nWDrw := 10;
      nHDrw := 10;
      nMod := 16;
      nMaxSpr := 256;
      nDSpr := 256;
      nCharSets := oROMsInfo.nCharSet;
    end;

    xHideMouse;

    { Draw the window border }
    xRectFill(nX, nY, nX+nW, nY+nH, VISIBLEPAGEOFFS, nLightGray);
    Border(nX, nY, nX+nW, nY+nH, VISIBLEPAGEOFFS, false);

    { Draw the caption area }
    xRectFill(nX+2, nY+2, nX+nW-2, nY+16-2, VISIBLEPAGEOFFS, nLightBlue);
    Border(nX+2, nY+2, nX+nW-2, nY+16-2, VISIBLEPAGEOFFS, true);
    if lSprite then
      OutTextXY(nX+8, nY+5, 'SPRITES', nYellow, nBlue, VISIBLEPAGEOFFS)
    else
      OutTextXY(nX+8, nY+5, 'CHAR SETS', nYellow, nBlue, VISIBLEPAGEOFFS);

    xShowMouse;
    MouseColor := nWhite;

    oButtons[1] := New(PButton, Init(1, nX+nW-64, nY+nH-26, 62, 20, kESC, 'Ok', oButtonSprite[OK_SPRITE]));
    oButtons[1]^.Draw;

    oButtons[2] := New(PCheckBox, Init(1, nX+nW-64, nY+20, 62, 16, kN, 1, 'No Flip',
      oButtonSprite[RADIO1_SPRITE], oButtonSprite[RADIO2_SPRITE]));
    oButtons[2]^.Draw;

    oButtons[3] := New(PCheckBox, Init(1, nX+nW-64, nY+40, 62, 16, kX, 0, 'Flip X',
      oButtonSprite[RADIO1_SPRITE], oButtonSprite[RADIO2_SPRITE]));
    oButtons[3]^.Draw;

    oButtons[4] := New(PCheckBox, Init(1, nX+nW-64, nY+60, 62, 16, kY, 0, 'Flip Y',
      oButtonSprite[RADIO1_SPRITE], oButtonSprite[RADIO2_SPRITE]));
    oButtons[4]^.Draw;

    oButtons[5] := New(PCheckBox, Init(1, nX+nW-64, nY+80, 62, 16, kF, 0, 'Flip XY',
      oButtonSprite[RADIO1_SPRITE], oButtonSprite[RADIO2_SPRITE]));
    oButtons[5]^.Draw;

    xLine(nX+nW-64, nY+80+20, nX+nW-4, nY+80+20, nWhite, VISIBLEPAGEOFFS);
    xLine(nX+nW-64, nY+80+21, nX+nW-4, nY+80+21, nDarkGray, VISIBLEPAGEOFFS);

    oButtons[6] := New(PSpinBox, Init(1, nX+nW-64, nY+104, 62, 20, kDn, 0, 15, 'Color'));
    oButtons[6]^.Draw;

    xLine(nX+nW-64, nY+104+22, nX+nW-4, nY+104+22, nWhite, VISIBLEPAGEOFFS);
    xLine(nX+nW-64, nY+104+23, nX+nW-4, nY+104+23, nDarkGray, VISIBLEPAGEOFFS);

    if nCharSets>1 then
      for i := 1 to nCharSets do begin
        oButtons[6+i] := New(PCheckBox, Init(1, nX+nW-64+28*(i-1), nY+130, 28, 10, k1, Byte((i-1)=nCharSet), sStrZero(i, 1),
          oButtonSprite[RADIO1_SPRITE], oButtonSprite[RADIO2_SPRITE]));
        oButtons[6+i]^.Draw;
      end;

    repeat
      lExit := false;
      { The type must be in 0..3 }
      if (nType>3) then nType := 3;
      if (nType<0) then nType := 0;

      { Draw the region of the screen where the sprites will appear }
      xHideMouse;
      xRectFill(nX+2, nY+16, nX+nW-2-64, nY+nH-2, VISIBLEPAGEOFFS, nBlack);
      Border(nX+2, nY+16, nX+nW-2-64, nY+nH-2, VISIBLEPAGEOFFS, true);

      { Draw all the sprites }
      for i := 0 to nMaxSpr-1 do begin
        case nType of
          0: xPutMaskedLBM(nX+04+(i mod nMod)*nWDrw,
                           nY+20+(i div nMod)*nHDrw,
                           VISIBLEPAGEOFFS,
                           nWSpr, nHSpr, 4*nColor,
                           P^[nCharSet*nDSpr+i]);
          1: xPutMaskedLBMFlipX(nX+04+(i mod nMod)*nWDrw,
                           nY+20+(i div nMod)*nHDrw,
                           VISIBLEPAGEOFFS,
                           nWSpr, nHSpr, 4*nColor,
                           P^[nCharSet*nDSpr+i]);
          2: xPutMaskedLBMFlipY(nX+04+(i mod nMod)*nWDrw,
                           nY+20+(i div nMod)*nHDrw,
                           VISIBLEPAGEOFFS,
                           nWSpr, nHSpr, 4*nColor,
                           P^[nCharSet*nDSpr+i]);
          3: xPutMaskedLBMFlipXY(nX+04+(i mod nMod)*nWDrw,
                           nY+20+(i div nMod)*nHDrw,
                           VISIBLEPAGEOFFS,
                           nWSpr, nHSpr, 4*nColor,
                           P^[nCharSet*nDSpr+i]);
        end;
      end;
      xShowMouse;


      { Verify the users response }
      repeat
        lRedraw := false;
        if oKBD.Key(kf6) then begin
          while oKBD.Key(kf6) do;
          SaveGIFProc;
        end;

        if nCharSets>1 then
          for i := 1 to nCharSets do
            if oButtons[6+i]^.CheckMessage then begin
              for j := 1 to nCharSets do
                oButtons[6+j]^.SetState(0);
              oButtons[6+i]^.SetState(1);
              nCharSet := i-1;
              lRedraw := true;
            end;

        for i := 1 to 6 do begin
          if oButtons[i]^.CheckMessage then begin
            case i of
              1: begin
                lExit := true;
                lRedraw := true;
                oButtons[i]^.SetState(0);
              end;
              2, 3, 4, 5: begin
                lRedraw := true;
                nType := i-2;
                oButtons[2]^.SetState(0);
                oButtons[3]^.SetState(0);
                oButtons[4]^.SetState(0);
                oButtons[5]^.SetState(0);
                oButtons[i]^.SetState(1);
              end;
              6: begin
                nColor := oButtons[6]^.GetValue;
                lRedraw := true;
              end;
            end;
          end;
        end;
      until lRedraw;
    until lExit;

    xHideMouse;
    for i := 1 to 6 do
      Dispose(oButtons[i], Done);
    if nCharSets>1 then
      for i := 1 to nCharSets do
        Dispose(oButtons[6+i], Done);
  end;

end.