{$O+}
unit
  UGalax;

interface

uses
  UZ80Var, UKBD, XLIB2, UGraph, UStrings, UZ80, XBM2, UDefault;

const
  GALAXIAN_IN0_PORT = $6000;
  GALAXIAN_IN0_SERVICE  =  (1 shl 7);
  GALAXIAN_IN0_TEST     =  (1 shl 6);
  GALAXIAN_IN0_TABLE    =  (1 shl 5);
  GALAXIAN_IN0_P1_SHOOT =  (1 shl 4);
  GALAXIAN_IN0_P1_RIGHT =  (1 shl 3);
  GALAXIAN_IN0_P1_LEFT  =  (1 shl 2);
  GALAXIAN_IN0_COIN2    =  (1 shl 1);
  GALAXIAN_IN0_COIN1    =  (1 shl 0);

  GALAXIAN_IN1_PORT     =  $6800;
  GALAXIAN_IN1_SW2      = (1 shl 7);
  GALAXIAN_IN1_SW1      = (1 shl 6);
  GALAXIAN_IN1_NOT_USED = (1 shl 5);
  GALAXIAN_IN1_P2_SHOOT = (1 shl 4);
  GALAXIAN_IN1_P2_RIGHT = (1 shl 3);
  GALAXIAN_IN1_P2_LEFT  = (1 shl 2);
  GALAXIAN_IN1_2P_START =  (1 shl 1);
  GALAXIAN_IN1_1P_START =  (1 shl 0);
  GALAXIAN_DSW_PORT     = $7000;
  GALAXIAN_DSW_SW6      = (1 shl 3);
  GALAXIAN_DSW_SW5      = (1 shl 2);
  GALAXIAN_DSW_SW4      = (1 shl 1);
  GALAXIAN_DSW_SW3      = (1 shl 0);
  GALAXIAN_END_OF_ROM   = $2800;
  GALAXIAN_NMI          = $7001;
  GALAXIAN_STARSON      = $7004;
  MAX_STARS = 250;
  nGFXBank: Integer = 0;
  nTotalStars: Integer = 0;
  nStarsScroll: Word = 0;

  function GalaxianGetByte(nEnder: Word): Byte;
  procedure GalaxianPutByte(nEnder, nValue: Word);
  procedure GalaxianInterrupt;
  procedure GalaxianConvertColorPROM(cFileName: String);


implementation

var
  aStars: array[0..MAX_STARS-1] of record
    x,y,code,col: Integer;
  end;


function GalaxianGetByte(nEnder: Word): Byte;
var
  RV: Byte;
begin

  if nEnder=GALAXIAN_IN0_PORT then begin
    rv := 0;
    {$ifndef DEBUG}
    if oKBD.Key(klf)   then rv  := rv or GALAXIAN_IN0_P1_LEFT;
    if oKBD.Key(krt)   then rv  := rv or GALAXIAN_IN0_P1_RIGHT;
    if oKBD.Key(k3)    then rv  := rv or GALAXIAN_IN0_COIN1;
    if oKBD.Key(k4)    then rv  := rv or GALAXIAN_IN0_COIN2;
    if oKBD.Key(kctrl) then rv  := rv or GALAXIAN_IN0_P1_SHOOT;
    if oRomsInfo.nDrvTag=2 then begin
      if oKBD.Key(kdn)   then rv  := rv or GALAXIAN_IN0_TEST;
      if oKBD.Key(kup)   then rv  := rv or GALAXIAN_IN0_SERVICE;
    end
    else begin
      if oKBD.Key(kf2)   then rv  := rv or GALAXIAN_IN0_TEST;
      if oKBD.Key(kf3)   then rv  := rv or GALAXIAN_IN0_SERVICE;
    end;
    {$endif}
    GalaxianGetByte := rv;
    exit
  end;

  if nEnder=GALAXIAN_IN1_PORT then begin
    rv := $10;
    {$ifndef DEBUG}
    if oKBD.Key(k2) then rv  := rv or GALAXIAN_IN1_2P_START;
    if oKBD.Key(k1) then rv  := rv or GALAXIAN_IN1_1P_START;
    {$endif}
    GalaxianGetByte := rv;
    exit
  end;

  if nEnder=GALAXIAN_DSW_PORT then begin
    rv := $00;
    GalaxianGetByte := $FF;
    exit
  end;



  GalaxianGetByte := aZ80Mem^[nEnder];
end;


procedure GalaxianPutByte(nEnder, nValue: Word);
begin
  if (nEnder<GALAXIAN_END_OF_ROM) then exit;
  if (nEnder=$6000) or (nEnder=$6001) then exit;

  if nEnder>=$4FFF then begin
    if aZ80Mem^[nEnder]<>nValue then begin
      lScreenChanged := true;
    end;
  end;

  if nEnder=$6002 then begin
    if oRomsInfo.nDrvTag>=$50 then begin
      nGFXBank := nValue and 1;
    end;
  end;

  aZ80Mem^[nEnder] := nValue
end;


procedure GalaxianInterrupt;
var
  nChar, nSprite, nEnder, i, j, nty, x, y, c, nDeltaY: LongInt;
  nY, nSX, nSY, nShift, nColor, nTotalX, nOffSetValue, nScanLine, nPos: LongInt;
  nFlip: Byte;
begin
  Inc(nStarsScroll);

  if true then begin

    xRectFill(0, 0, GetMaxX, GetMaxY, HIDDENPAGEOFFS, nBLACK);
    if nXPos<=8 then
      Border(nXPos-2, nYPos-2, nXPos+32*8+2, nYPos+32*8+2, HIDDENPAGEOFFS, false);
    nColor := 0;

    { Can I draw the stars? }
    if aZ80Mem^[oROMSInfo.nStarsOn]<>0 then
      for nOffSetValue := 0 to nTotalStars-1 do begin
        x := (aStars[nOffSetValue].x + nStarsScroll div 2) mod 256;
        y := aStars[nOffSetValue].y;
        if ((y and 1) xor ((x shr 4) and 1))<>0 then
          xPutPix(nXPos+y, nYPos+x, HiddenPageOffs, aStars[nOffSetValue].col)
      end;

    for y := 0 to 31 do begin
      if oROMSInfo.lInvertY then begin
        nY := (31-Y);
      end
      else begin
        nY := Y;
      end;
      nSY := nY*8;
      nShift := aZ80Mem^[oROMSInfo.nAttrRAM+2*nY];

      nColor := (aZ80Mem^[oROMSInfo.nAttrRAM+2*nY+1] and 7)*4;

      for x := 0 to 31 do begin

        if oROMSInfo.lInvertX then
          nOffSetValue := aZ80Mem^[oROMSInfo.nVRAM+(31-x)*32+nY]
        else
          nOffSetValue := aZ80Mem^[oROMSInfo.nVRAM+x*32+nY];
        if nOffSetValue=16 then continue;
        nSX := (x*8+nShift) mod 256;

        if nOffSetValue<>16 then
          xPutMaskedLBM(nXPos+nSX, nYPos+nSY, HiddenPageOffs, 8, 8, nColor, aLBMChars[nOffSetValue+nGFXBank*256]);
      end;
    end;

    for i := 0 to 7 do begin
      x := aZ80Mem^[oROMSInfo.nBulletRAM+4*i+1];
      if (x<>0) and (x<>255) then begin
        y := 256-aZ80Mem^[oROMSInfo.nBulletRAM+4*i+3]-4;
        for j := 0 to 2 do
          xPutPix(nXPos+x, nYPos+y+j, HiddenPageOffs, nWhite)
      end;
    end;


    for i := 0 to 7 do begin
      x := aZ80Mem^[oROMSInfo.nSpriteRAM+4*i];
      if (x>0) and (x<(255-32)) then begin
        nSprite := aZ80Mem^[oROMSInfo.nSpriteRAM+4*i+1] and $3F;
        nSprite := nSprite+nGFXBank*64;
        nColor := 4*(aZ80Mem^[oROMSInfo.nSpriteRAM+4*i+2] and 07);
        if nSprite<>0 then begin
          y := aZ80Mem^[oROMSInfo.nSpriteRAM+4*i+3];
          if (y>0) and (y<255) then begin
            nFlip := 0;
            if (aZ80Mem^[oROMSInfo.nSpriteRAM+4*i+1] and $40)<>0 then Inc(nFlip);
	    if (aZ80Mem^[oROMSInfo.nSpriteRAM+4*i+1] and $80)<>0 then Inc(nFlip, 2);
            case nFlip of
              0: xPutMaskedLBM(nXPos+X, nYPos+Y, HiddenPageOffs, 16, 16, nColor, aSprites[nSprite]);
              1: xPutMaskedLBMFlipY(nXPos+X, nYPos+Y, HiddenPageOffs, 16, 16, nColor, aSprites[nSprite]);
              2: xPutMaskedLBMFlipX(nXPos+X, nYPos+Y, HiddenPageOffs, 16, 16, nColor, aSprites[nSprite]);
              3: xPutMaskedLBMFlipXY(nXPos+X, nYPos+Y, HiddenPageOffs, 16, 16, nColor, aSprites[nSprite]);
            end;
          end;
        end;
      end;
    end;

    lScreenChanged := false;
  end;

  if aProcs[1].nPendingNMI=0 then begin
    aProcs[1].nPendingNMI := 0;
    PChar(aProcs[1].nPendingIRQ)[3] := #01;
    if oROMSInfo.nNMIEnable<>0 then begin
      if aZ80Mem^[oROMSInfo.nNMIEnable]<>0 then begin
        aProcs[1].nPendingNMI := 1;
      end
      else begin
        aProcs[1].nPendingIRQ := $01000000;
      end;
    end;
  end;
  NullInterrupt;
end;


procedure GalaxianConvertColorPROM(cFileName: String);
var
  nBits: Integer;
  nGenerator: LongInt;
  S, X, Y, I, nBit0, nBit1, nBit2, nVal: Integer;
  F: File;
  aPROM: PByteArray;
begin
  {$I-}
  Assign(F, cFileName);
  Reset(F, 1);
  {$I+}
  if IOResult<>0 then begin
    Writeln('Failed to load the color PROM '+cFileName);
    Halt(1);
  end;
  S := FileSize(F);
  GetMem(aPROM, S);
  BlockRead(F, aPROM^, S);
  Close(F);

  for i := 0 to 31 do begin
    nVal := aPROM^[i];
    nBit0 := (nVal shr 0) and $01;
    nBit1 := (nVal shr 1) and $01;
    nBit2 := (nVal shr 2) and $01;
    aPalette[i].R := $21*nBit0+$47*nBit1+$97*nBit2;

    nBit0 := (nVal shr 3) and $01;
    nBit1 := (nVal shr 4) and $01;
    nBit2 := (nVal shr 5) and $01;
    aPalette[i].G := $21*nBit0+$47*nBit1+$97*nBit2;

    nBit0 := 0;
    nBit1 := (nVal shr 6) and $01;
    nBit2 := (nVal shr 7) and $01;
    aPalette[i].B := $21*nBit0+$47*nBit1+$97*nBit2;
  end;

  for i := 0 to 63 do begin
    nBits :=  (i shr 0) and $03;
    aPalette[i+32].R := aPROM^[32+nBits];
    nBits :=  (i shr 2) and $03;
    aPalette[i+32].G := aPROM^[32+nBits];
    nBits :=  (i shr 4) and $03;
    aPalette[i+32].B := aPROM^[32+nBits];
  end;

  aPalette[4].R := 0; aPalette[4].G := 0; aPalette[4].B := $FF;


  nTotalStars := 0;
  nGenerator := 0;
  for y := 255 downto 0 do
    for x := 511 downto 0 do begin
      nGenerator := nGenerator shl 1;
      nBit1 := ((not nGenerator) shr 17) and 1;
      nBit2 := (nGenerator shr 5) and 1;
      if (nBit1 xor nBit2)<>0 then begin
        nGenerator := nGenerator or 1;
        if (y>=0) and (y<=255) and ((((not nGenerator) shr 16) and 1)<>0) and ((nGenerator and $FF)=$FF) then begin
          I := not(nGenerator shr 8) and $3F;
          if (I and nTotalStars)<MAX_STARS then begin
            aStars[nTotalStars].x := x;
            aStars[nTotalStars].y := y;
            aStars[nTotalStars].code := i;
            aStars[nTotalStars].col := i+32;
            Inc(nTotalStars);
          end;
        end;
      end;
    end;
  FreeMem(aPROM, S);
end;

end.