unit ULoadROM;

interface

uses
  UStrings;

type
  PPlaneInfo = ^TPlaneInfo;
  TPlaneInfo = array[1..1024] of Integer;

  TPort = record
    nType: Byte;
    nFunc: Byte;
    cStr: String[16];
  end;

  TPortInfo = record
    nAddr: Word;
    nNumber, nDefault: Byte;
    cName: String[10];
    oPort: array[0..7] of TPort;
  end;
  APortInfo = array[1..100] of TPortInfo;
  PPortInfo = ^APortInfo;

  TRomInfo = record
    cRomName: String;
    nLocation: LongInt;
  end;
  ARomInfo = array[1..100] of TRomInfo;
  PRomInfo = ^ARomInfo;

  TRomsInfo = object
    cColorProm,
    cTitle,
    cDriver: String[80];
    nMemory,
    nStarsOn,
    nDrvTag,
    nShiftAddr,
    nNMIEnable,
    nVW, nVH,
    nNMIInt,
    nMaxRom,
    nMaxDirectRead,
    nAttrRam,
    nSpriteRam,
    nBulletRam,
    nVRam: Word;
    nGRFDecode: Word;
    nCountRom: Integer;
    oRomInfo: PRomInfo;
    nPlanes,
    nCharSet,
    nSprites: Integer;
    oPlanesInfo: PPlaneInfo;
    nInPort: Integer;
    oInPort: PPortInfo;
    nError: Integer;
    lShift,
    lInvertY,
    lInvertX: Boolean;
    cErrorStr1, cErrorStr2: String[80];
    constructor LoadInfo(cFileName: String);
    procedure LoadRoms(P1, P2: Pointer);
    destructor Free;
  end;

  function HexToInt(S: String): LongInt;

implementation


 function HexToInt(S: String): LongInt;
 const
   cHexDig: String[16] = '0123456789ABCDEF';
 var
   nMult,
   nAux: LongInt;
   I, nPos: Integer;
 begin
   S := AllTrim(S);
   nAux := 0;
   nMult := 1;
   for i := Length(S) downto 1 do begin
     nPos := Pos(S[i], cHexDig);
     if nPos<>0 then
       nAux := nAux+(nPos-1)*nMult;
     nMult := nMult*16;
   end;
   HexToInt := nAux;
 end;


constructor TROMsInfo.LoadInfo(cFileName: String);
var
  F: System.Text;
  cAux,
  cLine: String;
  nPos: Integer;
  I, J: Integer;
  nPorta,
  nBit: Integer;
begin

  Assign(F, cFileName);
  Reset(F);
  nMemory := 65530;
  nVRam := $FFFF;
  nNMIEnable := $FFFF;
  nVW := $FFFF;
  nVH := $FFFF;
  nNMIInt := $0066;
  nMaxRom := $0000;
  nMaxDirectRead := $0000;
  nError := 0;
  nDrvTag := 0;
  nGRFDecode := $5000;
  cColorPROM := '';

  lInvertX := false;
  lInvertY := false;
  lShift := false;
  nCountRom := 0;
  nInPort := 0;
  while not Eof(F) do begin
    Readln(F, cLine);
    if cLine='' then Continue;


    if cLine[1] in ['0'..'9', 'A'..'F'] then
      Inc(nCountRom);

    if Copy(cLine, 1, 8)='#INVERTX' then lInvertX := true;
    if Copy(cLine, 1, 8)='#INVERTY' then lInvertY := true;
    if Copy(cLine, 1, 7)='#SHIFT=' then begin
      lShift := true;
      nShiftAddr := HexToInt(Copy(cLine, 8, 255));
    end;

    if Copy(cLine, 1, 8)='#INPORT=' then begin
      Inc(nInPort);
    end;

    if Copy(cLine, 1, 6)='#VRAM=' then begin
      nVRam := HexToInt(Copy(cLine, 7, 255));
    end;
    if Copy(cLine, 1, 8)='#DRVTAG=' then begin
      nDrvTag := HexToInt(Copy(cLine, 9, 255));
    end;
    if Copy(cLine, 1, 8)='#MEMORY=' then begin
      nMemory := HexToInt(Copy(cLine, 9, 255));
    end;
    if Copy(cLine, 1, 11)='#GRFDECODE=' then begin
      nGRFDecode := HexToInt(Copy(cLine, 12, 255));
    end;
    if Copy(cLine, 1, 11)='#BULLETRAM=' then begin
      nBulletRAM := HexToInt(Copy(cLine, 12, 255));
    end;
    if Copy(cLine, 1, 9)='#ATTRRAM=' then begin
      nAttrRAM := HexToInt(Copy(cLine, 10, 255));
    end;
    if Copy(cLine, 1, 11)='#SPRITERAM=' then begin
      nSpriteRAM := HexToInt(Copy(cLine, 12, 255));
    end;
    if Copy(cLine, 1, 8)='#NMIINT=' then begin
      nNMIInt := HexToInt(Copy(cLine, 9, 255));
    end;
    if Copy(cLine, 1, 9)='#STARSON=' then begin
      nStarsOn := HexToInt(Copy(cLine, 10, 255));
    end;
    if Copy(cLine, 1, 8)='#MAXROM=' then begin
      nMaxRom := HexToInt(Copy(cLine, 9, 255));
    end;
    if Copy(cLine, 1, 15)='#MAXDIRECTREAD=' then begin
      nMaxDirectRead := HexToInt(Copy(cLine, 16, 255));
    end;
    if Copy(cLine, 1, 8)='#VIDEOW=' then begin
      nVW := HexToInt(Copy(cLine, 9, 255));
    end;
    if Copy(cLine, 1, 8)='#VIDEOH=' then begin
      nVH := HexToInt(Copy(cLine, 9, 255));
    end;
    if Copy(cLine, 1, 11)='#NMIENABLE=' then begin
      nNMIEnable := HexToInt(Copy(cLine, 12, 255));
    end;
    if Copy(cLine, 1, 7)='#TITLE=' then begin
      cTitle := Copy(cLine, 8, 255);
    end;
    if Copy(cLine, 1, 8)='#PLANES=' then begin
      nPlanes := HexToInt(Copy(cLine, 9, 255));
    end;
    if Copy(cLine, 1, 9)='#SPRITES=' then begin
      nSprites := HexToInt(Copy(cLine, 10, 255));
    end;
    if Copy(cLine, 1, 9)='#CHARSET=' then begin
      nCharSet := HexToInt(Copy(cLine, 10, 255));
    end;
    if Copy(cLine, 1, 8)='#DRIVER=' then begin
      cDriver := Copy(cLine, 9, 255);
    end;
    if Copy(cLine, 1, 11)='#COLORPROM=' then begin
      cColorPROM := Copy(cLine, 12, 255);
    end;
  end;
  Close(F);

  GetMem(oRomInfo, nCountRom*SizeOf(TRomInfo));
  GetMem(oInPort, nInPort*SizeOf(TPortInfo));

  GetMem(oPlanesInfo, nPlanes*SizeOf(oPlanesInfo^[1]));
  for i := 1 to nInPort do
    for j := 0 to 7 do
      with oInPort^[i].oPort[j] do begin
        nType := 0;
        nFunc := 0;
        cStr := ''
      end;

  nCountRom := 0;
  nInPort := 0;
  Assign(F, cFileName);
  Reset(F);
  while not Eof(F) do begin
    Readln(F, cLine);
    if cLine='' then Continue;
    if Copy(cLine, 1, 12)='#PLANOFFSET=' then begin
      cAux := Copy(cLine, 13, 255);
    end;

    if Copy(cLine, 1, 8)='#INPORT=' then begin
      Inc(nInPort);
      with oInPort^[nInPort] do begin
        nAddr := HexToInt(Copy(cLine, 12, 04));
        nNumber := HexToInt(Copy(cLine, 09, 02));
        nDefault := HexToInt(Copy(cLine, 17, 02));
        cName := Copy(cLine, 20, 255);
      end;
    end;

    if Copy(cLine, 1, 10)='#INPVALUE=' then begin
      nPorta := HexToInt(Copy(cLine, 11, 02));
      if (nPorta<1) or (nPorta>nInPort) then begin
        nError := 1;
        cErrorStr1 := 'Invalid Port Value: '+Copy(cLine, 11, 02);
        cErrorStr2 := cLine;
      end;

      nBit := HexToInt(Copy(cLine, 16, 01));
      if (nBit<0) or (nBit>7) then begin
        nError := 1;
        cErrorStr1 := 'Invalid bit number: '+Copy(cLine, 16, 01);
        cErrorStr2 := cLine;
      end;

      with oInPort^[nPorta].oPort[nBit] do begin
        nType := HexToInt(Copy(cLine, 14, 01));
        nFunc := HexToInt(Copy(cLine, 18, 02));
        cStr := Copy(cLine, 21, 255);
      end;
    end;

    if cLine[1] in ['0'..'9', 'A'..'F'] then begin
      Inc(nCountRom);
      nPos := Pos('~', cLine);
      if nPos<>0 then begin
        oRomInfo^[nCountRom].nLocation := HexToInt(Copy(cLine, 1, nPos-1));
        oRomInfo^[nCountRom].cRomName := Copy(cLine, nPos+1, 255);
      end;
    end;
  end;
  Close(F);

  if nVRAM=$FFFF then nVRAM := $4000;
  if nNMIEnable=$FFFF then nNMIEnable := $7001;
  if nVH=$FFFF then nVH := 32;
  if nVW=$FFFF then nVW := 29;
  Dec(nVH);
  Dec(nVW);
end;

procedure TRomsInfo.LoadRoms(P1, P2: Pointer);
type
  TByteArray = array[0..1024] of byte;
  PByteArray = ^TByteArray;
var
  cFileName: String;
  F: File;
  PAux: PChar;
  I, J: Integer;
begin
  for i := 1 to nCountRom do begin
    PAux := P1;
    cFileName := oRomInfo^[i].cRomName;
    {$I-}
    Assign(F, cFileName);
    Reset(F, 1);
    {$I+}
    if IOResult<>0 then begin
      Writeln('Error loading ROMs.');
      Writeln('The following ROM was not found: '+cFileName);
      Writeln('I need to load the following ROMs:');
      for j := 1 to nCountROM do
        Writeln(oRomInfo^[j].cRomName);
      Writeln('Abnormal program termination.');
      Halt(1);
    end;
    BlockRead(F, PAux[oRomInfo^[i].nLocation], FileSize(F));
    Close(F);
  end;
end;

destructor TRomsInfo.Free;
begin
  FreeMem(oRomInfo, nCountRom*SizeOf(TRomInfo));
  FreeMem(oPlanesInfo, nPlanes*SizeOf(oPlanesInfo^[1]));
  FreeMem(oInPort, nInPort*SizeOf(TPortInfo));
end;

end.
