{=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Unit UObjects
  -=-=-=-=-=-=-
  Objects used in the emulator GUI
  TButton: Buttons
  TCheckBox: Check Box
  TListBox: ListBox
  TSpinBox: Box that contain number that may change in a range

  Author: Paulo Sergio Cardoso - April 8, 1998
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-}

unit
  UObjects;

interface
  Uses
    UKBD, Strings, UGraph;

  type
    SString = String[10];

    TStrArray = array [1..1] of PChar;
    PStrArray = ^TStrArray;

    PParent = ^TParent;
    TParent = object
      nId, nX, nY, nW, nH: Integer;
      nKey: ScanCode;
      cString: SString;
      constructor Init(aId, aX, aY, aW, aH: Integer; aKey: ScanCode; aString: SString);
      procedure Draw; virtual;
      function CheckMessage: boolean; virtual;
      function GetValue: Integer; virtual;
      procedure SetState(nNewState: Integer); virtual;
      destructor  Done;
    end;

    PButton = ^TButton;
    TButton = object(TParent)
      lState: Boolean;
      oLBMImage: PLBMImage;
      constructor Init(aId, aX, aY, aW, aH: Integer; aKey: ScanCode; aString: SString; aImage: PLBMImage);
      function CheckMessage: boolean; virtual;
      procedure SetState(nNewState: Integer); virtual;
      procedure Draw; virtual;
    end;

    PCheckBox = ^TCheckBox;
    TCheckBox = object(TParent)
      nState: Byte;
      nColor: Byte;
      oGr1, oGr2: PLBMImage;
      constructor Init(aId, aX, aY, aW, aH: Integer; aKey: ScanCode; aState: Byte; aString: SString; aGr1, aGr2: PLBMImage);
      function CheckMessage: boolean; virtual;
      procedure SetState(nNewState: Integer); virtual;
      procedure Draw; virtual;
    end;

    PSpinBox = ^TSpinBox;
    TSpinBox = object(TParent)
      nState: Byte;
      nMin, nMax: Integer;
      constructor Init(aId, aX, aY, aW, aH: Integer; aKey: ScanCode; aMin, aMax: Integer; aString: SString);
      function CheckMessage: boolean; virtual;
      procedure SetState(nNewState: Integer); virtual;
      procedure Draw; virtual;
      function GetValue: Integer; virtual;
    end;

    PListBox = ^TListBox;
    TListBox = object(TParent)
      nCount: Integer;
      oUp, oDn: PButton;
      oList: PStrArray;
      nBase, nQtdShow,
      nLastIndex, nIndex: Integer;
      lRedraw: boolean;
      constructor Init(aId, aX, aY, aW, aH: Integer; aKey: ScanCode);
      procedure Add(cStr: String);
      procedure Draw; virtual;
      function CheckMessage: Boolean; virtual;
      function GetString: String;
      destructor Done; virtual;
    end;

implementation
  uses
    XLIB2, UZ80Var, UStrings;

  constructor TParent.Init;
  begin
    nX := aX;
    nY := aY;
    nW := aW;
    nH := aH;
    nKey := aKey;
    cString := aString;
  end;

  procedure TParent.Draw;
  begin
  end;

  function TParent.CheckMessage: boolean;
  begin
  end;

  function TParent.GetValue: Integer;
  begin
  end;

  procedure TParent.SetState(nNewState: Integer);
  begin
  end;

  destructor TParent.Done;
  begin
  end;

  constructor TButton.Init;
  begin
    Inherited Init(aId, aX, aY, aW, aH, aKey, aString);
    lState := false;
    oLBMImage := aImage;
  end;

  procedure TButton.Draw;
  var
    nDelta: Integer;
  begin
    xHideMouse;
    xRectFill(nX, nY, nX+nW-1, nY+nH-1, VISIBLEPAGEOFFS, nLightGray);
    Border(nX, nY, nX+nW-1, nY+nH-1, VISIBLEPAGEOFFS, lState);
    nDelta := 0;
    if lState then Inc(nDelta);
    if oLBMImage=nil then
      ShadowPrint(nX+4+nDelta, nY+((nH-8) div 2)+nDelta, VISIBLEPAGEOFFS, nWhite, nDarkGray, cString)
    else begin
      if cString='' then begin
        nDelta := (nW-oLBMImage^.nW) div 2;
        if lState then Inc(nDelta)
      end
      else
        Inc(nDelta, 1);
      xPutMaskedLBM(nX+nDelta, nY+nDelta, VISIBLEPAGEOFFS,
        oLBMImage^.nW, oLBMImage^.nH, 0, @oLBMImage^.aData);
      ShadowPrint(nX+2+oLBMImage^.nW+nDelta, nY+((nH-8) div 2)+nDelta, VISIBLEPAGEOFFS, nWhite, nDarkGray, cString)
    end;
    xShowMouse;
  end;

  function TButton.CheckMessage: boolean;
  begin
    if (MouseButtonStatus<>0) and (MouseX>=nX) and (MouseX<=(nX+nW-1)) and
                                  (MouseY>=nY) and (MouseY<=(nY+nH-1)) then begin
      lState := true;
      Draw;
      CheckMessage := true;
      while MouseButtonStatus<>0 do;
      Exit;
    end;

    if oKBD.Key(nKey) then begin
      lState := true;
      Draw;
      CheckMessage := true;
      while oKBD.Key(nKey) do;
      Exit;
    end;

    CheckMessage := false;

  end;

  procedure TButton.SetState(nNewState: Integer);
  begin
    lState := nNewState<>0;
    Draw;
  end;


  constructor TCheckBox.Init;
  begin
    Inherited Init(aId, aX, aY, aW, aH, aKey, aString);
    nState := aState;
    oGr1 := aGr1;
    oGr2 := aGr2;
    xHideMouse;
    nColor := xGetPix(aX, aY, VISIBLEPAGEOFFS);
    xShowMouse;
  end;

  procedure TCheckBox.Draw;
  var
    I: Integer;
  begin
    xHideMouse;
    xRectFill(nX, nY, nX+nW-1, nY+nH-1, VISIBLEPAGEOFFS, nColor);
    ShadowPrint(nX+2+oGr1^.nW, nY+(nH-8) div 2, VISIBLEPAGEOFFS, nWhite, nDarkGray, cString);
    if nState=0 then begin
      xPutMaskedLBM(nX, nY, VISIBLEPAGEOFFS,
        16, 16, 0, @oGr1^.aData);
    end
    else begin
      xPutMaskedLBM(nX, nY, VISIBLEPAGEOFFS,
        16, 16, 0, @oGr2^.aData);
    end;

    {
    if nState=0 then begin
      xLine(nX+00, nY+05, nX+05, nY+00, nWhite, VISIBLEPAGEOFFS);
      xLine(nX+05, nY+00, nX+10, nY+05, nDarkGray, VISIBLEPAGEOFFS);
      xLine(nX+10, nY+05, nX+05, nY+10, nDarkGray, VISIBLEPAGEOFFS);
      xLine(nX+05, nY+10, nX+00, nY+05, nWhite, VISIBLEPAGEOFFS);
    end
    else begin
      for i := 0 to 3 do begin
        xLine(nX+2+3-i, nY+2+i, nX+2+3+i, nY+2+i, nRed, VISIBLEPAGEOFFS);
        xLine(nX+2+3-i, nY+2+6-i, nX+2+3+i, nY+2+6-i, nRed, VISIBLEPAGEOFFS);
      end;
      xLine(nX+00, nY+05, nX+05, nY+00, nDarkGray, VISIBLEPAGEOFFS);
      xLine(nX+05, nY+00, nX+10, nY+05, nWhite, VISIBLEPAGEOFFS);
      xLine(nX+10, nY+05, nX+05, nY+10, nWhite, VISIBLEPAGEOFFS);
      xLine(nX+05, nY+10, nX+00, nY+05, nDarkGray, VISIBLEPAGEOFFS);

      xLine(nX+2+00, nY+2+03, nX+2+03, nY+2+00, nWhite, VISIBLEPAGEOFFS);
      xLine(nX+2+03, nY+2+00, nX+2+06, nY+2+03, nDarkGray, VISIBLEPAGEOFFS);
      xLine(nX+2+06, nY+2+03, nX+2+03, nY+2+06, nDarkGray, VISIBLEPAGEOFFS);
      xLine(nX+2+03, nY+2+06, nX+2+00, nY+2+03, nWhite, VISIBLEPAGEOFFS);
    end;
    }
    xShowMouse;
  end;

  function TCheckBox.CheckMessage: boolean;
  begin
    if (MouseButtonStatus<>0) and (MouseX>=nX) and (MouseX<=(nX+nW-1)) and
                                  (MouseY>=nY) and (MouseY<=(nY+nH-1)) then begin
      if nState=0 then
        nState := 1
      else
        nState := 0;
      Draw;
      CheckMessage := true;
      while MouseButtonStatus<>0 do;
      Exit;
    end;


    if oKBD.Key(nKey) then begin
      if nState=0 then
        nState := 1
      else
        nState := 0;
      Draw;
      CheckMessage := true;
      while oKBD.Key(nKey) do;
      Exit;
    end;

    CheckMessage := false;

  end;

  procedure TCheckBox.SetState(nNewState: Integer);
  begin
    nState := nNewState;
    Draw;
  end;


  constructor TSpinBox.Init;
  begin
    Inherited Init(aId, aX, aY, aW, aH, aKey, aString);
    nMin := aMin;
    nMax := aMax;
  end;

  procedure TSpinBox.Draw;
  var
    I: Integer;
  begin
    xHideMouse;
    xRectFill(nX, nY, nX+nW-1, nY+nH-1, VISIBLEPAGEOFFS, nLightGray);
    xRectFill(nX+8, nY, nX+nW-1, nY+nH-1, VISIBLEPAGEOFFS, nYellow);
    Border(nX, nY, nX+nW-1, nY+nH-1, VISIBLEPAGEOFFS, true);
    Border(nX+1, nY+1, nX+8, nY+9, VISIBLEPAGEOFFS, false);
    Border(nX+1, nY+10, nX+8, nY+nH-2, VISIBLEPAGEOFFS, false);

    xPrintf(nX+2, nY+2, VISIBLEPAGEOFFS, nBlack, '-');
    xPrintf(nX+2, nY+11, VISIBLEPAGEOFFS, nBlack, '+');

    xPrintf(nX+12, nY+02, VISIBLEPAGEOFFS, nBlack, cString);
    xPrintf(nX+12, nY+10, VISIBLEPAGEOFFS, nBlack, AllTrim(sStr(nMin, 2))+'/'+AllTrim(sStr(nMax, 2)));
    xShowMouse;
  end;

  function TSpinBox.CheckMessage: boolean;
  begin
    if (MouseButtonStatus<>0) and (MouseX>=(nX+1)) and (MouseX<=(nX+8)) and
                                  (MouseY>=(nY+10)) and (MouseY<=(nY+nH-2)) then begin
      xHideMouse;
      Border(nX+1, nY+10, nX+8, nY+nH-2, VISIBLEPAGEOFFS, true);
      xShowMouse;
      while MouseButtonStatus<>0 do;
      Inc(nMin);
      if nMin>nMax then nMin := nMax;
      Draw;
      CheckMessage := true;
      Exit;
    end;

    if (MouseButtonStatus<>0) and (MouseX>=(nX+1)) and (MouseX<=(nX+8)) and
                                  (MouseY>=(nY+1)) and (MouseY<=(nY+9)) then begin
      xHideMouse;
      Border(nX+1, nY+1, nX+8, nY+9, VISIBLEPAGEOFFS, true);
      xShowMouse;
      while MouseButtonStatus<>0 do;
      Dec(nMin);
      if nMin<0 then nMin := 0;
      Draw;
      CheckMessage := true;
      Exit;
    end;

    if oKBD.Key(kDn) then begin
      Dec(nMin);
      if nMin<0 then nMin := 0;
      Draw;
      CheckMessage := true;
      while oKBD.Key(kDn) do;
      Exit;
    end;

    if oKBD.Key(kUp) then begin
      Inc(nMin);
      if nMin>nMax then nMin := nMax;
      Draw;
      CheckMessage := true;
      while oKBD.Key(kUp) do;
      Exit;
    end;

    CheckMessage := false;
  end;

  function TSpinBox.GetValue: Integer;
  begin
    GetValue := nMin;
  end;

  procedure TSpinBox.SetState(nNewState: Integer);
  begin
    nState := nNewState;
    Draw;
  end;

  constructor TListBox.Init;
  begin
    Inherited Init(aId, aX, aY, aW, aH, aKey, '');
    nCount := 0;
    nIndex := 1;
    oList := nil;
    nBase := 0;
    nQtdShow := (nH-4) div 10;
    oUp := New(PButton, Init(0, aX+nW-1-17, aY+1, 16, 16, kUp, '', oButtonSprite[UP_SPRITE]));
    oDn := New(PButton, Init(0, aX+nW-1-17, aY+nH-1-17, 16, 16, kDn, '', oButtonSprite[DOWN_SPRITE]));
    lRedraw := true;
  end;

  function TListBox.CheckMessage: Boolean;
  var
    I: Integer;
  begin
    if (MouseButtonStatus<>0) and (MouseX>=(nX+2)) and (MouseX<=(nX+nW-18)) and
                                  (MouseY>=(nY+2)) and (MouseY<=(nY+nH-2)) then begin
      I := nBase+1+(MouseY-nY-4) div 10;
      if (I>0) and (I<=nCount) then begin
        if (nIndex=i) then begin
          CheckMessage := true;
          Exit;
        end;
        nIndex := i;
        Draw;
      end;
      while MouseButtonStatus<>0 do;
    end;
    if oUp^.CheckMessage then begin
      oUp^.SetState(0);
      if nIndex>1 then begin
        Dec(nIndex);
        Draw;
      end;
    end;
    if oDn^.CheckMessage then begin
      oDn^.SetState(0);
      if nIndex<nCount then begin
        Inc(nIndex);
        Draw;
      end;
    end;

    if oKbd.Key(kHome) then begin
      if nIndex<>1 then begin
        nIndex := 1;
        Draw;
      end;
    end;

    if oKbd.Key(kEnd) then begin
      if nIndex<>nCount then begin
        nIndex := nCount;
        Draw;
      end;
    end;

    if oKbd.Key(kPgDn) then begin
      if nIndex<=(nCount-nQtdShow) then begin
        nIndex := nIndex+nQtdShow;
        Draw;
      end
      else
        nIndex := nCount;
      Draw;
      while oKbd.Key(kPgDn) do;
    end;

    if oKbd.Key(kPgUp) then begin
      if nIndex>=(nQtdShow+1) then begin
        nIndex := nIndex-nQtdShow;
        Draw;
      end
      else
        nIndex := 1;
      Draw;
      while oKbd.Key(kPgUp) do;
    end;
    CheckMessage := false;
  end;

  procedure TListBox.Draw;
  var
    nPosBar, nBarW, nColor, I, nMax, nPos: Integer;
    cAux: String;
  begin
    xHideMouse;

    if nIndex<(nBase+1) then begin
      nBase := nIndex-1;
      lRedraw := true;
    end;
    if nIndex>(nBase+nQtdShow) then begin
      nBase := nIndex-nQtdShow;
      lRedraw := true;
    end;

    if lRedraw then begin
      xRectFill(nX, nY, nX+nW-1, nY+nH-1, VISIBLEPAGEOFFS, nLightGray);
      Border(nX, nY, nX+nW-1, nY+nH-1, VISIBLEPAGEOFFS, true);
      Border(nX+nW-1-17, nY+17, nX+nW-3, nY+nH-2-17, VISIBLEPAGEOFFS, true);

      nBarW := nH-2-34-2;
      nPosBar := 0;
      if nQtdShow<nCount then begin
        nBarW := Round(nBarW*nQtdShow/nCount);
        nPosBar := Round((nH-2-34-2-nBarW)*nBase/(nCount-nQtdShow));
      end;

      Border(nX+nW-1-17+1, nY+17+1+nPosBar,
             nX+nW-3-1, nY+17+1+nPosBar+nBarW,
             VISIBLEPAGEOFFS, false);

      nMax := nBase+nQtdShow;
      if nMax>nCount then nMax := nCount;
      for i := nBase+1 to nMax do begin
        if nIndex=i then begin
          nColor := nWhite;
          xRectFill(nX+2, nY+2+(i-nBase-1)*10,
                    nX+nW-18, nY+2+(i-nBase-1)*10+10,
                    VISIBLEPAGEOFFS, nBlack);
        end
        else begin
          nColor := nYellow;
        end;
        cAux := StrPas(oList^[i]);
        nPos := Pos(#1, cAux);
        cAux := Copy(cAux, 1, nPos-1);
        xPrintf(nX+5, nY+4+(i-nBase-1)*10, VISIBLEPAGEOFFS, nDarkGray, cAux);
        xPrintf(nX+4, nY+3+(i-nBase-1)*10, VISIBLEPAGEOFFS, nColor, cAux);
        lRedraw := false;
      end;
    end
    else begin
      I := nLastIndex;
      nColor := nYellow;
      xRectFill(nX+2, nY+2+(i-nBase-1)*10,
                nX+nW-18, nY+2+(i-nBase-1)*10+10,
                VISIBLEPAGEOFFS, nLightGray);
      cAux := StrPas(oList^[i]);
      nPos := Pos(#1, cAux);
      cAux := Copy(cAux, 1, nPos-1);
      xPrintf(nX+5, nY+4+(i-nBase-1)*10, VISIBLEPAGEOFFS, nDarkGray, cAux);
      xPrintf(nX+4, nY+3+(i-nBase-1)*10, VISIBLEPAGEOFFS, nColor, cAux);

      I := nIndex;
      nColor := nWhite;
      xRectFill(nX+2, nY+2+(i-nBase-1)*10,
                nX+nW-18, nY+2+(i-nBase-1)*10+10,
                VISIBLEPAGEOFFS, nBlack);
      cAux := StrPas(oList^[i]);
      nPos := Pos(#1, cAux);
      cAux := Copy(cAux, 1, nPos-1);
      xPrintf(nX+5, nY+4+(i-nBase-1)*10, VISIBLEPAGEOFFS, nDarkGray, cAux);
      xPrintf(nX+4, nY+3+(i-nBase-1)*10, VISIBLEPAGEOFFS, nColor, cAux);
    end;

    nLastIndex := nIndex;

    xShowMouse;
    oUp^.Draw;
    oDn^.Draw;
  end;

  procedure TListBox.Add(cStr: String);
  var
    oNewList: PStrArray;
    I, J: Integer;
    P: Pointer;
  begin
    cStr := cStr+#0;
    GetMem(oNewList, SizeOf(oList^[1])*(nCount+1));
    if nCount<>0 then begin
      for i := 1 to nCount do
        oNewList^[i] := oList^[i];
      FreeMem(oList, SizeOf(oList^[1])*nCount);
    end;
    Inc(nCount);
    oList := oNewList;
    oList^[nCount] := StrNew(@cStr[1]);

    for i := 1 to nCount-1 do begin
      if StrPas(oList^[nCount])<StrPas(oList^[i]) then begin
        P := oList^[nCount];
        for j := nCount downto i+1 do
          oList^[j] := oList^[j-1];
        oList^[i] := P;
        break;
      end;
    end;
    lRedraw := true;
    Draw;
  end;

  function TListBox.GetString: String;
  var
    cAux: String;
    nPos: Integer;
  begin
    cAux := StrPas(oList^[nIndex]);
    nPos := Pos(#1, cAux);
    Delete(cAux, 1, nPos);
    GetString := cAux;
  end;

  destructor TListBox.Done;
  var
    I: Integer;
  begin
    if nCount<>0 then begin
      for i := 1 to nCount do
        StrDispose(oList^[i]);
      FreeMem(oList, SizeOf(oList^[1])*nCount);
    end;
    inherited Done;
  end;

end.
